#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>

#include "path/to/libdvbpsi/config.h"

#include <dvbpsi/dvbpsi.h>
#include <dvbpsi/psi.h>
#include <dvbpsi/descriptor.h>
#include <dvbpsi/nit.h>

static void message(dvbpsi_t *handle, const dvbpsi_msg_level_t level, const char *msg) {

    switch(level) {
        case DVBPSI_MSG_ERROR:
            fprintf(stderr, "%s", msg);
            break;
        case DVBPSI_MSG_WARN:
            fprintf(stderr, "%s", msg);
            break;
        case DVBPSI_MSG_DEBUG:
            fprintf(stdout, "%s", msg);
            break;
        default:
            return;
    }
}

static void write_psi(dvbpsi_psi_section_t *section, const uint16_t pid, uint8_t *packet, FILE *fp) {

    packet[0] = 0x47;

    while (section) {
        size_t bytes_written = 0;
        uint8_t *position_in_ts;
        uint8_t *byte = section->p_data;
        uint8_t *end  = section->p_payload_end + (section->b_syntax_indicator ? 4 : 0);

        packet[1] |= 0x40 | (pid >> 8);
        packet[2] = pid & 0xFF;
        packet[3] = (packet[3] & 0x0F) | 0x10;

        packet[4] = 0x00;
        position_in_ts = packet + 5;

        while (position_in_ts < packet + 188 && byte < end)
            *(position_in_ts++) = *(byte++);
        while (position_in_ts < packet + 188)
            *(position_in_ts++) = 0xFF;

        bytes_written = fwrite(packet, 188, 1, fp);
        if (bytes_written == 0)
            return;

        packet[3] = (packet[3] + 1) & 0x0F;

        while (byte < end) {
            packet[1] &= 0xBF;
            packet[3] = (packet[3] & 0x0F) | 0x10;

            position_in_ts = packet + 4;

            while (position_in_ts < packet + 188 && byte < end)
                *(position_in_ts++) = *(byte++);
            while (position_in_ts < packet + 188)
                *(position_in_ts++) = 0xFF;
            bytes_written = fwrite(packet, 188, 1, fp);
            if (bytes_written == 0)
                return;

            packet[3] = (packet[3] + 1) & 0x0F;
        }

        section = section->p_next;
    }
}

int main(void) {

    FILE *fp;
    uint8_t packet[188] = { 0 };

    dvbpsi_t *dvbpsi = dvbpsi_new(&message, DVBPSI_MSG_DEBUG);
    dvbpsi_nit_t *nit = dvbpsi_nit_new(0x40, 666, 1337, 1, true);
    dvbpsi_psi_section_t *sections = NULL;

    dvbpsi_nit_ts_add(nit, 666, 1337);

    fp = fopen("out.ts", "wb");
    if (fp == NULL)
        goto out;

    sections = dvbpsi_nit_sections_generate(dvbpsi, nit, 0x40);
    write_psi(sections, 0x10, packet, fp);
    fclose(fp);

out:
    dvbpsi_nit_delete(nit);
    dvbpsi_DeletePSISections(sections);
    dvbpsi_delete(dvbpsi);

    return 0;
}

